@@ -1,5 +1,6 @@
 import sys
 from argparse import Namespace
+from collections.abc import MutableMapping
 from enum import auto
 from fnmatch import fnmatch
 from itertools import zip_longest
@@ -14,6 +15,7 @@ from strenum import LowercaseStrEnum
 
 from nef_pipelines.lib import util
 from nef_pipelines.lib.constants import NEF_PIPELINES
+from nef_pipelines.lib.globals_lib import set_global
 from nef_pipelines.lib.util import (
     exit_error,
     fixup_metadata,
@@ -24,6 +26,13 @@ from nef_pipelines.lib.util import (
     script_name,
 )
 
+NEF_PIPELINES_PREFIX = "nefpls"
+
+NEF_RELAXATION_VERSION = "0.1.0a"
+
+DEFAULT_INCREMENT_SEPARATOR = "::"
+DEFAULT_INCREMENT_END = ""
+
 NEF_TRUE = "true"
 NEF_FALSE = "false"
 NEF_NONE = "none"
@@ -33,6 +42,7 @@ NEF_MOLECULAR_SYSTEM = "nef_molecular_system"
 NEF_METADATA = "nef_nmr_meta_data"
 
 UNUSED = "."
+UNDERSCORE = "_"
 
 # this stops pynmrstar failing on empty strings
 STR_CONVERSION_DICT[""] = UNUSED
@@ -134,11 +144,30 @@ POTENTIAL_TYPES_LOWER = ", ".join(
 #     return loop
 
 
+def get_frame_id(frame: Saveframe) -> str:
+    """
+    given a frame get its id (the characters after the category and a separating _ charctacter)
+    :param frame: the frame to get the name of
+    :return: the id of the frame
+    """
+    return frame.name[len(frame.category) + 1 :]
+
+
+def get_frame_ids(frames_or_entry: Union[List[Saveframe], Entry]) -> List[str]:
+    """
+    given a list of frames get theirs id (the characters after the category and a separating _ charctacter)
+    :param frames_or_entry: a list of frames or an entty
+    :return: a list of ids in the same order
+    """
+
+    return list([get_frame_id(frame) for frame in frames_or_entry])
+
+
 def select_frames_by_name(
     frames: Union[List[Saveframe], Entry],
     name_selectors: Union[List[str], str],
     exact=False,
-) -> Tuple[Saveframe]:
+) -> Tuple[Saveframe, ...]:
     """
     select frames  by names and wild cards, to avoid typing *s on the command line the match is greedy by default,
     if an exact match is not found for one of the frames first time we search again with all the name selectors
@@ -153,7 +182,7 @@ def select_frames_by_name(
 
     def match_frames(
         frames: List[Saveframe], name_selectors: List[str]
-    ) -> List[Saveframe]:
+    ) -> Dict[Any, Saveframe]:
         result = {}
 
         for frame in frames:
@@ -183,7 +212,6 @@ def select_frames_by_name(
 
 # refactor to two functions one of which gets a TextIO
 def create_entry_from_stdin() -> Optional[Entry]:
-
     """
     read a star file entry from stdin or return None
 
@@ -206,18 +234,22 @@ def create_entry_from_stdin() -> Optional[Entry]:
     except ParsingError as e:
         raise BadNefFileException(str(e)) from e
 
+    if entry:
+        entry.source = "-"
+
     return entry
 
 
 def read_file_or_exit(file_path: Path) -> List[str]:
     """
-    read the contenst of a file or exit with error
+    read the contents of a file or exit with error
 
     :param file_path: the path to the file
     :return: a list of strings from the file
     """
     try:
-        result = open(file_path).readlines()
+        with open(file_path) as fh:
+            result = fh.readlines()
     except IOError as e:
         msg = f"""\
             failed to read from {file_path} because {e}
@@ -229,7 +261,6 @@ def read_file_or_exit(file_path: Path) -> List[str]:
 
 # refactor to two functions one of which gets a TextIO
 def read_entry_from_stdin_or_exit() -> Entry:
-
     """
     read a star file entry from stdin or exit with an error message
     :return: a star file entry
@@ -238,9 +269,6 @@ def read_entry_from_stdin_or_exit() -> Entry:
     if sys.stdin.isatty():
         exit_error("you appear to be reading from an empty stdin")
 
-    if running_in_pycharm():
-        exit_error("streaming doesn't work inside pycharm")
-
     try:
         stdin_lines = sys.stdin.read()
         if stdin_lines is None:
@@ -266,10 +294,69 @@ def read_entry_from_stdin_or_exit() -> Entry:
             e,
         )
 
+    entry.source = "-"
+
     return entry
 
 
+class RowDict(MutableMapping):
+    def __init__(self, loop: Loop, row: int, convert: bool):
+        super().__init__()
+        self._data = row
+        self._convert = convert
+        self._tag_to_index = {tag: i for i, tag in enumerate(loop.tags)}
+        self._tag_types = {
+            tag: type(value) for tag, value in zip(loop.tags, self._data)
+        }
+
+    def __len__(self):
+        return len(self._data)
+
+    def __iter__(self):
+        return iter(self._tag_to_index.keys())
+
+    def __delitem__(self, key):
+        self._data[self._tag_to_index[key]] = UNUSED
+
+    def __getitem__(self, key):
+        index = self._tag_to_index[key]
+        value = self._data[index]
+        if self._convert:
+            value = do_reasonable_type_conversions(value)
+        return value
+
+    def __setitem__(self, key, value):
+        index = self._tag_to_index[key]
+        if self._tag_types[key] == str and not isinstance(value, str):
+            value = str(value)
+        self._data[index] = value
+
+    def __repr__(self):
+        row = {key: self._data[index] for key, index in self._tag_to_index.items()}
+        return str(row)
+
+    def __str__(self):
+        return self.__repr__()
+
+    # def __getattribute__(self, item):
+    #     print(item)
+    #     try:
+    #         return self[item]
+    #     except Exception:
+    #         return object.__getattribute__(self, item)
+    #
+    # def __setattr__(self, item, value):
+    #     if hasattr(self, item):
+    #         self[item] = value
+    #     else:
+    #         return super().__setattr__(item, value)
+    #
+    # def __delattr__(self, key):
+    #     self.__delitem__(key)
+
+
 # TODO we should examine columns for types not individual rows entries
+# TODO we should rename this loop_row_iter
 def loop_row_dict_iter(
     loop: Loop, convert: bool = True
 ) -> Iterator[Dict[str, Union[str, int, float]]]:
@@ -288,14 +375,9 @@ def loop_row_dict_iter(
         """
         raise Exception(msg)
 
-    for row in loop:
-        row = {tag: value for tag, value in zip(loop.tags, row)}
-
-        if convert:
-            for key in row:
-                row[key] = do_reasonable_type_conversions(row[key])
-
-        yield row
+    for i, row in enumerate(loop):
+        result = RowDict(loop, row, convert)
+        yield result
 
 
 def do_reasonable_type_conversions(value: str) -> Union[str, float, int]:
@@ -322,25 +404,35 @@ def loop_row_namespace_iter(loop: Loop, convert: bool = True) -> Iterator[Namesp
     :param loop: thr Loop
     :param convert: try to convert values to ints or floats if possible [default is True]
     :return: iterator of rows as dictionaries
+
+    NOTE: This function is effectively replaced by loop_row_dict_iter, which returns a RowDict object
+          with support for attribute lookup
     """
     for row in loop_row_dict_iter(loop, convert=convert):
+        # yield row needs more tests modified
         yield Namespace(**row)
 
 
 # TODO this partially overlaps with select_frames_by_name in this file, combine and simplify!
 def select_frames(
-    entry: Entry, selector_type: SelectionType, filters: List[str]
+    entry: Entry,
+    filters: Union[str, List[str]],
+    selector_type: SelectionType = SelectionType.ANY,
 ) -> List[Saveframe]:
     """
     select a list of frames by name of either category or name
 
     :param entry: the entry in which frames are looked for
+    :param filters: a string or list of strings to use as filters as defined by fnmatch
     :param selector_type: the matching type frame.name or frame.category or both [default, search order
            frame.name frame.category]
-    :param filters: a list of strings to use as filters as defined by fnmatch
     :return: a list of selected saveframes
     """
 
+    if isinstance(filters, str):
+        filters = [
+            filters,
+        ]
     if not filters:
         filters = ["*"]
 
@@ -385,12 +477,29 @@ def read_entry_from_file_or_stdin_or_exit_error(file: Path) -> Entry:
     if file is None or file == Path("-"):
         entry = read_entry_from_stdin_or_exit()
     else:
-        try:
-            with open(file) as fh:
-                entry = Entry.from_file(fh)
+        entry = read_entry_from_file_or_exit_error(file)
+    return entry
+
+
+def read_entry_from_file_or_exit_error(file):
+    """
+    read a star entry from a file or exit with an error message
+
+    :param file: a file on the file system
+    :return: a new Entry read from the file
+    """
+
+    file = Path(file)
+
+    if not file.exists() or not file.is_file():
+        exit_error(f"the file {file} doesn't exist or isn't a file")
+    try:
+        with open(file) as fh:
+            entry = Entry.from_file(fh)
+
+    except IOError as e:
+        exit_error(f"couldn't read from the file {file}", e)
 
-        except IOError as e:
-            exit_error(f"couldn't read from the file {file}", e)
     return entry
 
 
@@ -409,9 +518,13 @@ def read_or_create_entry_exit_error_on_bad_file(
     try:
         if file is None or file == Path("-"):
             entry = create_entry_from_stdin()
+            if entry and entry.entry_id == "nefpls_globals":
+                entry.entry_id = entry_name
         else:
             try:
-                entry = Entry.from_file(file.open())
+                with open(file) as fh:
+                    entry = Entry.from_file(fh)
+                    _parse_globals(entry)
 
             except IOError as e:
                 msg = f"""
@@ -432,6 +545,20 @@ def read_or_create_entry_exit_error_on_bad_file(
     return entry
 
 
+def _parse_globals(entry: Entry):
+
+    globals_frames = entry.get_saveframes_by_category("nefpls_globals")
+    for globals_frame in globals_frames:
+
+        values = {name: globals_frame.get_tag(name) for name in globals_frame.tags}
+        values = {
+            name: do_reasonable_type_conversions(value)
+            for name, value in values.items()
+        }
+        for key, value in values.items():
+            set_global(key, value)
+
+
 def file_name_path_to_frame_name(path: str) -> str:
     """
     convert a filename or complete path to a name suitable for use as a frame name, this does the following mappings
@@ -534,7 +661,7 @@ def is_save_frame_name_in_entry(entry: Entry, frame_name: str) -> bool:
     :param frame_name: the frame name
     :return: if a savefame with the name is present
     """
-    frame_in_entry = False
+    frame_in_entry = True
     try:
         entry.get_saveframe_by_name(frame_name)
     except KeyError:
@@ -607,3 +734,33 @@ def extract_column(loop: Loop, column: Union[str, int]) -> Loop:
     for row in loop:
         values.append(row[column])
     return values
+
+
+def create_nef_save_frame(
+    frame_category: str,
+    frame_id: str = None,
+    increment: str = "",
+    separator: str = DEFAULT_INCREMENT_SEPARATOR,
+    end: str = DEFAULT_INCREMENT_END,
+) -> Saveframe:
+    """
+    Create a NEF save frame from a category and a frame name, including the tags sf_category and sf_framecode
+
+    :param category: the category including an organisation prefix
+    :param frame_id: the id of the frame  (can be None for singletons)
+    :param separator: separator between the category and the frame_id (default ::)
+    :param end: text after the frame_id (can be used to create a fence  default is the empty string
+    :return: a NEF style saveframe
+    """
+
+    if frame_id and increment:
+        frame_name = f"{frame_category}_{frame_id}{separator}{increment}{end}"
+    elif frame_id:
+        frame_name = f"{frame_category}_{frame_id}"
+    else:
+        frame_name = f"{frame_category}"
+
+    frame = Saveframe.from_scratch(frame_name, f"{frame_category}")
+    frame.add_tag("sf_category", frame_category)
+    frame.add_tag("sf_framecode", frame_name)
+    return frame
