@@ -1,10 +1,11 @@
+import os
 import string
 from collections import Counter
 from dataclasses import replace
 from enum import auto
 from pathlib import Path
 from textwrap import dedent
-from typing import Dict, Iterable, List, Optional, Tuple, Union
+from typing import Any, Dict, Iterator, List, Optional, Tuple, Union
 
 from ordered_set import OrderedSet
 from pynmrstar import Entry, Loop, Saveframe
@@ -21,6 +22,7 @@ from nef_pipelines.lib.util import (
     get_display_file_name,
     is_int,
     read_from_file_or_exit,
+    strip_characters_right,
 )
 
 
@@ -35,11 +37,44 @@ NEF_CHAIN_CODE = "chain_code"
 ANY_CHAIN = "_"
 
 
-def chain_code_iter(
+class ChainCodeIterable:
+
+    ASCII_UPPERCASE = list(string.ascii_uppercase)
+
+    def __init__(self, user_chain_codes: List[str], exclude: List[str] = ()):
+
+        self._index = 0
+        self._chain_codes = []
+
+        seen_codes = set()
+        self._chain_codes = []
+        for user_chain_code in user_chain_codes:
+            seen_codes.add(user_chain_code)
+            if user_chain_code not in exclude:
+                self._chain_codes.append(user_chain_code)
+
+        for chain_code in self.ASCII_UPPERCASE:
+            if chain_code not in seen_codes and chain_code not in exclude:
+                self._chain_codes.append(chain_code)
+
+    def __iter__(self):
+        return self
+
+    def __next__(self):
+        if self._index < len(self._chain_codes):
+            result = self._chain_codes[self._index]
+            self._index += 1
+            return result
+        else:
+            raise StopIteration("run out of chain names!")
+
+
+def get_chain_code_iter(
     user_chain_codes: List[str] = "", exclude: List[str] = ()
-) -> Iterable[str]:
+) -> Iterator[str]:
     """
-    Yield chain codes in user chain codes and once exhausted yield any remaining letters of the upper case alphabet
+    return an iterator they will yield chain codes in user chain codes and
+    once exhausted yield any remaining letters of the upper case alphabet
     till they run out. A list of chain codes to not use can be provided...
 
     Args:
@@ -50,20 +85,7 @@ def chain_code_iter(
         Iterable[str] single codes
     """
 
-    ascii_uppercase = list(string.ascii_uppercase)
-
-    seen_codes = set()
-
-    for chain_code in user_chain_codes:
-        seen_codes.add(chain_code)
-        if chain_code not in exclude:
-            yield chain_code
-
-    for chain_code in ascii_uppercase:
-        if chain_code not in seen_codes and chain_code not in exclude:
-            yield chain_code
-
-    raise ValueError("run out of chain names!")
+    return ChainCodeIterable(user_chain_codes, exclude).__iter__()
 
 
 def get_linking(
@@ -71,7 +93,6 @@ def get_linking(
     no_chain_start=List[str],
     no_chain_end=List[str],
 ) -> List[SequenceResidue]:
-
     """
     get the correct linking for residue in a standard sequenced chain
 
@@ -149,13 +170,18 @@ def sequence_to_nef_frame(
 
     for index, (sequence_residue, linking) in enumerate(residue_and_linkages):
 
-        nef_loop.add_data_by_tag("index", index + 1)
-        nef_loop.add_data_by_tag("chain_code", sequence_residue.chain_code)
-        nef_loop.add_data_by_tag("sequence_code", sequence_residue.sequence_code)
-        nef_loop.add_data_by_tag("residue_name", sequence_residue.residue_name.upper())
-        nef_loop.add_data_by_tag("linking", linking)
-        nef_loop.add_data_by_tag("residue_variant", NEF_UNKNOWN)
-        nef_loop.add_data_by_tag("cis_peptide", NEF_UNKNOWN)
+        data = [
+            {
+                "index": index + 1,
+                "chain_code": sequence_residue.chain_code,
+                "sequence_code": sequence_residue.sequence_code,
+                "residue_name": sequence_residue.residue_name.upper(),
+                "linking": linking,
+                "residue_variant": NEF_UNKNOWN,
+                "cis_peptide": NEF_UNKNOWN,
+            }
+        ]
+        nef_loop.add_data(data)
 
     return nef_frame
 
@@ -224,7 +250,37 @@ class BadResidue(Exception):
     Bad residue found in a sequence
     """
 
-    pass
+    def __init__(self, residue_name, error_pos, sequence_string):
+
+        self.residue_name = residue_name
+        self.sequence_string = sequence_string
+        self.error_pos = error_pos
+
+    def __str__(self):
+        try:
+            terminal_width = os.get_terminal_size().columns
+        except OSError:
+            terminal_width = 80
+
+        error_pos_string = [
+            " ",
+        ] * len(self.sequence_string)
+        error_pos_string[self.error_pos] = "^"
+        error_pos_string = "".join(error_pos_string)
+
+        msg = f"""\
+        unknown residue {self.residue_name}
+        at residue {self.error_pos+1}
+        sequence:
+        """
+        msg = dedent(msg)
+        sequence_string_chunks = list(chunks(self.sequence_string, terminal_width - 10))
+        pos_string_chunks = list(chunks(error_pos_string, terminal_width - 10))
+        for sequence_chunk, pos_chunk in zip(sequence_string_chunks, pos_string_chunks):
+            msg += f"{''.join(sequence_chunk)}\n"
+            msg += f"{''.join(pos_chunk)}\n"
+
+        return msg
 
 
 def translate_1_to_3(
@@ -247,6 +303,9 @@ def translate_1_to_3(
 
     result = []
     for i, residue_name_1let in enumerate(sequence):
+        if residue_name_1let == " ":
+            continue
+        original_residue_name_1let = residue_name_1let
         residue_name_1let = residue_name_1let.upper()
         if translations is None:
             result.append(residue_name_1let)
@@ -256,13 +315,7 @@ def translate_1_to_3(
             if unknown:
                 result.append(unknown)
             else:
-                msg = f"""\
-                     unknown residue {residue_name_1let} at residue {i+1}
-                     sequence: {sequence}
-                               {(' ' * i) + '^'}
-                """
-                msg = dedent(msg)
-                raise BadResidue(msg)
+                raise BadResidue(original_residue_name_1let, i, sequence)
 
     return result
 
@@ -340,7 +393,7 @@ def sequence_residues_to_sequence_3let(
     Translate a list of SequenceResidues to 3 letter sequence codes t
     Args:
         sequence (List([SequenceResidues]): list of residues
-        chain_code (str): the chain code [defaults to A] only residues fromthis chain will be converted
+        chain_code (str): the chain code [defaults to A] only residues from this chain will be converted
 
     Returns List[str]:
         the sequence as a list of 3 letter residues
@@ -461,13 +514,12 @@ def sequence_from_entry(entry) -> List[SequenceResidue]:
 def sequence_from_frame(
     frame: Saveframe, chain_codes_to_select: Union[str, List[str]] = ANY_CHAIN
 ) -> List[SequenceResidue]:
-
     """
     read sequences from a nef molecular system save frame
     can raise Exceptions
 
     :param frame: the save frame to read residues from, must have category nef molecular system
-    :param chain_code_to_select: the chain codes to select,t this can be either a string or list of strings,
+    :param chain_code_to_select: the chain codes to select this can be either a string or list of strings,
                                  any chain is selected using the instance of the constant string ANY_CHAIN [default]
     :return: a list of parsed residues and chains, in the order they were read
     """
@@ -497,22 +549,37 @@ def sequence_from_frame(
     sequence_code_index = loop.tag_index("sequence_code")
     residue_name_index = loop.tag_index("residue_name")
     linking_index = loop.tag_index("linking")
+    residue_variant_index = loop.tag_index("residue_variant")
 
     for line in loop:
         chain_code = line[chain_code_index]
         if chain_codes_to_select is ANY_CHAIN or chain_code in chain_codes_to_select:
             sequence_code = line[sequence_code_index]
+            sequence_code = (
+                int(sequence_code) if is_int(sequence_code) else sequence_code
+            )
             residue_name = line[residue_name_index]
             linking = (
                 Linking[line[linking_index].upper()]
                 if line[linking_index] != NEF_UNKNOWN
                 else None
             )
+            residue_variants = line[residue_variant_index].split(",")
+            residue_variants = (
+                ()
+                if residue_variants
+                == [
+                    UNUSED,
+                ]
+                else tuple(residue_variants)
+            )
+
             residue = SequenceResidue(
                 chain_code=chain_code,
                 sequence_code=sequence_code,
                 residue_name=residue_name,
                 linking=linking,
+                variants=residue_variants,
             )
             residues.append(residue)
 
@@ -527,7 +594,7 @@ def sequence_3let_to_res(
     :param sequence_3_let: 3 letter names
     :param chain_code: chain code to use
     :param start:  start of the chain [default is 1]
-    :return: a list of sequeence residues
+    :return: a list of sequence residues
     """
     result = set()
     for res_number, residue_name in enumerate(sequence_3_let, start=start):
@@ -541,7 +608,7 @@ def get_chain_starts(residues: List[SequenceResidue]) -> Dict[str, int]:
     from a list of residues get the lowest residue number for each chain ignoring any sequence codes that
     can't be converted to an integer
 
-    :param residues:  a list of residues from one ore more chains
+    :param residues:  a list of residues from one or more chains
     :return: a dictionary of chain starts by chain_code
     """
 
@@ -564,7 +631,7 @@ def sequence_to_chains(residues: List[SequenceResidue]) -> List[str]:
     """
     from a list of residues get chain_codes
 
-    :param residues:  a list of residues from one ore more chains
+    :param residues:  a list of residues from one or more chains
     :return: a list of chain_codes
     """
 
@@ -584,7 +651,7 @@ def offset_chain_residues(
     take a list of residues and offset residue sequence_codes by any offsets in chains if the
     sequence_code can be converted to an integer and the chain_code matches
 
-    :param residues:  a list of residues from one ore more chains
+    :param residues:  a list of residues from one or more chains
     :return: a list of residues
     """
 
@@ -603,11 +670,35 @@ def offset_chain_residues(
     return result
 
 
+def sequence_3let_list_from_sequence(
+    sequence: List[SequenceResidue], count: int = 10, chain_code="A"
+) -> List[str]:
+    """
+    convert a list of Residues to a set of lines containing space separated residue names
+    [default 10] per line
+
+    :param sequence: a sequence
+    :param count: number of residues names to display per line
+    :return: a list of lines as strings
+    """
+
+    residue_names = [
+        residue.residue_name for residue in sequence if residue.chain_code == chain_code
+    ]
+    rows = chunks(residue_names, count)
+
+    row_strings = []
+    for row in rows:
+        row_strings.append(" ".join(row))
+
+    return row_strings
+
+
 def make_chunked_sequence_1let(
     sequence_1_let: List[str], sub_chunk: int = 10, line_length: int = 100
 ) -> List[str]:
     """
-    convert a list of strings into a chunked list of strings with (by de]fault) every 100 sting being
+    convert a list of strings into a chunked list of strings with (by default) every 100 sting being
     separated by a new line [line_length] and sequences of strings being separated by a space
     :param sequence_1_let: a list of strings typically 1 single letter
     :param sub_chunk: how often to separate strings by a space
@@ -615,31 +706,17 @@ def make_chunked_sequence_1let(
     :return: a list of formatted strings one per line
     """
 
-    rows = chunks(sequence_1_let, 100)
+    rows = chunks(sequence_1_let, line_length)
 
     row_strings = []
     for row in rows:
-        row_chunks = list(chunks(row, 10))
+        row_chunks = list(chunks(row, sub_chunk))
         row_string = ["".join(chunk) for chunk in row_chunks]
         row_strings.append(" ".join(row_string))
 
     return row_strings
 
 
-def residues_to_residue_name_lookup(
-    sequence: List[SequenceResidue],
-) -> Dict[Tuple[str, str], str]:
-
-    result = None
-    if len(sequence) > 0:
-        result = {
-            (residue.chain_code, str(residue.sequence_code)): residue.residue_name
-            for residue in sequence
-        }
-
-    return result
-
-
 def replace_chain_in_atom_labels(
     atom_labels: List[AtomLabel], chain_code: str
 ) -> List[AtomLabel]:
@@ -663,7 +740,7 @@ def get_residue_name_from_lookup(
     chain_code: str,
     sequence_code: Union[str, int],
     lookup_table: Dict[Union[str, int], str],
-) -> str:
+) -> Optional[str]:
     """
     get a residue name from a chain_code and sequence_code, using the provided lookup table. The sequence code is
     treated being as either  a string or an int, strs are checked first
@@ -689,16 +766,115 @@ def get_residue_name_from_lookup(
     return residue_name
 
 
-def sequence_to_residue_type_lookup(
+def sequence_to_residue_name_lookup(
     sequence: List[SequenceResidue],
 ) -> Dict[Tuple[str, int], str]:
-
     """
     build a lookup table from sequence_code, chain_code to residue_type from a list of sequence residues
     :param sequence: a list of sequence residues
     :return: the lookup table
     """
+
     result: Dict[Tuple[str, int], str] = {}
-    for residue in sequence:
-        result[residue.chain_code, residue.sequence_code] = residue.residue_name
+    if len(sequence) > 0:
+        result = {
+            (residue.chain_code, str(residue.sequence_code)): residue.residue_name
+            for residue in sequence
+        }
+
     return result
+
+
+def atom_sort_key(item: AtomLabel) -> Tuple[Any, ...]:
+    """
+
+    key based sorter for atom labels that enables them to be sorted in a sensible order including sequence codes
+    which are not numeric...
+
+    :param item: an atom label
+    :return: a tuple based on the atom label which gives the correct sort order
+    """
+
+    try:
+        items = []
+        residue = item.residue
+        chain_code = residue.chain_code.strip()
+        prefix, numbers = strip_characters_right(chain_code, string.digits)
+
+        if numbers == "":
+            numbers = 0, ""
+        if is_int(numbers):
+            numbers = int(numbers), ""
+        else:
+            numbers = 0, numbers
+
+        if UNUSED in prefix:
+            order = 2
+        elif "@" in prefix or "#" in prefix:
+            order = 1
+        else:
+            order = 0
+
+        items.append([order, prefix, *numbers])
+
+        sequence_code = str(residue.sequence_code).strip()
+        prefix, numbers = strip_characters_right(sequence_code, (string.digits + "+-"))
+
+        if is_int(numbers):
+            numbers = [int(numbers), "", 0]
+        elif "+" in numbers or "-" in numbers:
+            for splitter in "+-":
+                if splitter in numbers:
+                    numbers = numbers.rsplit(splitter, maxsplit=1)
+                    numbers = [numbers[0], splitter, numbers[1]]
+                    break
+            if is_int(numbers[0]) and is_int(numbers[-1]):
+                numbers[0] = int(numbers[0])
+                numbers[-1] = int(numbers[-1])
+        else:
+            numbers = [0, numbers, 0]
+
+        if UNUSED in prefix:
+            order = 2
+        elif "@" in prefix or "#" in prefix:
+            order = 1
+        else:
+            order = 0
+
+        items.append([order, prefix, *numbers])
+
+        items.append(residue.residue_name.lower())
+        items.append(item.atom_name.lower())
+    except Exception as e:
+        msg = f"""
+            I can't sort {item} because it doesn't seem to be behaving like an AtomLabel...
+            [{e}]
+        """
+        raise ValueError(msg)
+
+    return items
+
+
+def exit_if_chain_not_in_entrys_sequence(chain_code: str, entry: Entry):
+
+    nef_sequence = sequence_from_entry(entry)
+
+    _exit_if_chain_not_in_sequence(chain_code, nef_sequence, entry)
+
+
+def _exit_if_chain_not_in_sequence(
+    chain_code: str,
+    nef_sequence: List[SequenceResidue],
+    entry: Entry,
+):
+    file_name = f"{entry.source} [stdin]" if entry.source == "-" else entry.source
+
+    nef_chain_codes = sequence_to_chains(nef_sequence)
+    if chain_code and chain_code not in nef_chain_codes:
+        msg = f"""
+            The chain code {chain_code} was not found in the input chain codes in the nef stream
+            with entry name {entry.entry_id}  in file {file_name}
+            the chain codes found were {','.join(nef_chain_codes)}
+        """
+
+        exit_error(msg)
