@@ -1,18 +1,31 @@
 import contextlib
+import functools
 import inspect
 import io
 import os
 import sys
 import traceback
+import warnings
 from argparse import Namespace
 from enum import auto
+from fnmatch import fnmatch
 from math import floor
 from pathlib import Path
 from textwrap import dedent
-from typing import Any, Dict, Iterator, List, Optional, TextIO, Tuple, TypeVar, Union
+from typing import (
+    Any,
+    Dict,
+    Iterable,
+    Iterator,
+    List,
+    Optional,
+    TextIO,
+    Tuple,
+    TypeVar,
+    Union,
+)
 
 import click
-from cacheable_iter import iter_cache
 from pynmrstar import Entry, Loop, Saveframe
 from strenum import StrEnum
 from tabulate import tabulate
@@ -23,6 +36,7 @@ from nef_pipelines.lib.constants import (
     NEF_PIPELINES,
     NEF_UNKNOWN,
 )
+from nef_pipelines.lib.globals_lib import get_global
 from nef_pipelines.lib.header_lib import (
     create_header_frame,
     get_creation_time,
@@ -30,11 +44,15 @@ from nef_pipelines.lib.header_lib import (
 )
 from nef_pipelines.lib.structures import LineInfo
 
+UNKNOWN_INPUT_SOURCE = "unknown"
+
 FOUR_SPACES = " " * 4
 
 STDIN = Path("-")
 STDOUT = Path("-")
 
+NEWLINE = "\n"
+
 
 def _get_loop_by_category_or_none(frame: Saveframe, category: str) -> Loop:
 
@@ -75,16 +93,21 @@ def fixup_metadata(entry: Entry, name: str, version: str, script: str):
     if entry is not None and NEF_META_DATA in entry.category_list:
         meta_frame = entry[NEF_META_DATA]
 
-        last_program = meta_frame.get_tag("program_name")[0]
-        last_program_version = meta_frame.get_tag("program_version")[0]
-        last_script_name = meta_frame.get_tag("script_name")[0]
+        last_program = meta_frame.get_tag("program_name")
+        last_program = last_program[0] if last_program else "."
+
+        last_program_version = meta_frame.get_tag("program_version")
+        last_program_version = last_program_version[0] if last_program_version else "."
+
+        last_script_name = meta_frame.get_tag("script_name")
+        last_script_name = last_script_name[0] if last_script_name else "."
 
         meta_frame.add_tag("program_name", name, update=True)
         meta_frame.add_tag("program_version", version, update=True)
         meta_frame.add_tag("script_name", script, update=True)
-        creation_time = get_creation_time()
-        meta_frame.add_tag("creation_time", creation_time, update=True)
-        uuid = get_uuid(NEF_PIPELINES, creation_time)
+        creation_date = get_creation_time()
+        meta_frame.add_tag("creation_date", creation_date, update=True)
+        uuid = get_uuid(NEF_PIPELINES, creation_date)
         meta_frame.add_tag("uuid", uuid, update=True)
 
         run_history_loop = _get_loop_by_category_or_none(meta_frame, "nef_run_history")
@@ -209,7 +232,8 @@ def get_text_from_file_or_exit(file_name: Path) -> str:
         result = sys.stdin.read()
     else:
         try:
-            result = open(file_name).read()
+            with open(file_name) as fh:
+                result = fh.read()
         except IOError as e:
             exit_error(f"couldn't read from {file_name} because of an error", e)
 
@@ -229,8 +253,10 @@ def get_pipe_file(args: Namespace) -> Optional[TextIO]:
     """
 
     result = None
-    if "pipe" in args and args.pipe:
-        result = iter(cached_file_stream(args.pipe))
+    if "input" in args and args.input and args.input != STDIN:
+        with open(args.input) as fh:
+            pipe_lines = fh.read()
+        result = io.StringIO(pipe_lines)
     # pycharm doesn't treat stdstreams correcly and hangs
     elif not sys.stdin.isatty() and not running_in_pycharm():
         result = iter(sys.stdin.read().split("\n"))
@@ -260,69 +286,8 @@ def get_pipe_file_text(args: Namespace) -> Optional[str]:
     return result if result else None
 
 
-def get_pipe_file_text_or_exit(args: Namespace) -> Optional[TextIO]:
-
-    try:
-        result = get_pipe_file_text(args)
-    except Exception as e:
-        exit_error("couldn't read from stdin or -pipe file", e)
-
-    if result is None:
-        exit_error("couldn't read from stdin and no -pipe in command line arguments")
-
-    return result
-
-
-def get_pipe_file_or_exit(args: Namespace) -> Optional[TextIO]:
-
-    try:
-        result = get_pipe_file(args)
-    except Exception as e:
-        exit_error("couldn't read from stdin or -pipe file", e)
-
-    if isinstance(result, StringIteratorIO):
-
-        result = "\n".join(list(result))
-
-        if len(result) == 0:
-            result = None
-        else:
-            result = io.StringIO(result)
-
-    if result is None:
-        exit_error("couldn't read from stdin and no -pipe in command line arguments")
-
-    return result
-
-
-@iter_cache
-def cached_stdin():
-    # TODO: this doesn't work!'
-    return sys.stdin
-
-
-class cached_file_stream:
-    def __init__(self, file_name):
-        self._file_name = file_name
-
-    def __enter__(self):
-        return _cached_file_stream(self._file_name)
-
-    def __exit__(self, *args):
-        pass
-
-    def __iter__(self):
-        return _cached_file_stream(self._file_name)
-
-
-@iter_cache
-def _cached_file_stream(file_name):
-    try:
-        with open(file_name, "r") as lines:
-            result = lines.readlines()
-    except IOError as e:
-        exit_error(f"couldn't open stream {file_name} because {e}")
-    return result
+def in_pytest():
+    return "pytest" in sys.modules
 
 
 def script_name(file: str) -> Path:
@@ -341,8 +306,7 @@ def script_name(file: str) -> Path:
 
         current_context = Namespace(command_path="unknown unknown")
 
-    in_pytest = "pytest" in sys.modules
-    if in_pytest:
+    if in_pytest():
         command = current_context.command_path.split()
     else:
         command = current_context.command_path.split()[1:]
@@ -352,24 +316,23 @@ def script_name(file: str) -> Path:
     return command
 
 
-def read_from_file_or_exit(file_name: Path, target: str) -> str:
-    if file_name == STDIN and sys.stdin.isatty():
-        exit_error(
-            f"trying to open stdin to read {target}, but stdin is not a stream [did you forget to add a nef file to "
-            "your pipeline?]"
-        )
-    if running_in_pycharm():
-        exit_error(
-            "reading from stdin doesn't work in pycharm debug environment as there is no shell..."
-        )
+def read_from_file_or_exit(file_path: Path, target: str = UNKNOWN_INPUT_SOURCE) -> str:
 
-    display_file_name = get_display_file_name(file_name)
+    if file_path == STDIN and sys.stdin.isatty():
+
+        msg = """
+            while trying to read from STDIN [-] the input is not a stream
+            [did you forget to add a nef file to your pipeline?]
+        """
+        exit_error(msg)
+
+    display_file_name = get_display_file_name(file_path)
 
     fh = sys.stdin
 
-    if not file_name == STDIN:
+    if not file_path == STDIN:
         try:
-            fh = open(file_name)
+            fh = open(file_path)
         except Exception as e:
             msg = f"couldn't open {display_file_name} to read {target}"
             exit_error(msg, e)
@@ -380,6 +343,9 @@ def read_from_file_or_exit(file_name: Path, target: str) -> str:
         msg = f"couldn't read {target} from {display_file_name}"
         exit_error(msg, e)
 
+    if file_path != STDIN:
+        fh.close()
+
     return text
 
 
@@ -402,6 +368,7 @@ def exit_error(msg, exception=None):
         msg: the message
     """
 
+    # This should only happen in verbose mode
     if exception is not None:
         exc_info = sys.exc_info()
         traceback.print_exception(*exc_info, file=sys.stderr)
@@ -445,7 +412,7 @@ def process_stream_and_add_frames(
         exit_error(f"failed to load pipe file because {e}")
 
     if stream is not None:
-        lines = "\n".join(stream.readlines())
+        lines = stream.read()
     else:
         lines = ""
 
@@ -510,6 +477,16 @@ def is_float(value: str) -> bool:
 T = TypeVar("T")
 
 
+def nef_pipelines_root():
+    """
+    get the root of the nef pipelines installation
+
+    Returns:
+        the root of the nef pipelines installation
+    """
+    return Path(__file__).parent.parent.parent
+
+
 # https://stackoverflow.com/questions/312443/how-do-i-split-a-list-into-equally-sized-chunks
 def chunks(input: Iterator[T], n: int) -> Iterator[List[T]]:
     """Yield successive n-sized chunks from lst.
@@ -657,7 +634,7 @@ def smart_open(filename=None, stream_type=StdStream.STDOUT):
     f"""
     context manager to open a file or use stdout depending on the filename, stdin is defined as '-'
     by corollary a std stream won't be closed on exit only file system streams
-    :param filename: the filename to use '-' indicates a standar stream selected by stream_type
+    :param filename: the filename to use '-' indicates a standard stream selected by stream_type
     :param stream_type: how to treat the stream choices are {', '.join(StdStream.__members__)}
 
     """
@@ -847,7 +824,6 @@ def strings_to_tabulated_terminal_sensitive(
 
 
 def strip_characters_left(target: str, letters: str) -> Tuple[str, str]:
-
     """
      strip the characters from letters from the left of the target and return a tuple containing
         1. the string without the stripped letters
@@ -892,3 +868,173 @@ def strip_line_comment(line: str, comment_character: str = "#") -> Tuple[str, st
         line = line[:index]
 
     return line, comment
+
+
+# https://stackoverflow.com/questions/480214/how-do-i-remove-duplicates-from-a-list-while-preserving-order
+def remove_duplicates_stable(seq: Iterable) -> List:
+    """
+    remove  duplicates from a list while preseving its order, so for example
+    [1 2 2 3 45 2 1 6] would yield 1 2 3 4 5 6
+    :param seq: the iteratble to remove duplicates from
+    :return: the original iterable without the duplicates
+
+    TODO: could this be recast as an iterable returning an iterable
+    """
+
+    seen = set()
+    seen_add = seen.add
+    return [x for x in seq if not (x in seen or seen_add(x))]
+
+
+# https://stackoverflow.com/questions/21303224/iterate-over-all-pairs-of-consecutive-items-in-a-list
+def iter_consecutive_pairs(seq: Iterator) -> Iterator[Tuple]:
+    """
+    iterate over consecutive pairs from a list, for example for the list [1, 7, 3, 5]
+    this will yield the items (1,7), (7,3), (3,5)
+    :param seq: the sequence to iterate pairs from
+    :return: an iteration over the consecutive pairs as tuples
+    """
+    for first, second in zip(seq, seq[1:]):
+        yield first, second
+
+
+# https://stackoverflow.com/questions/2536307/decorators-in-the-python-standard-lib-deprecated-specifically
+string_types = (type(b""), type(""))
+
+
+def deprecated(reason):
+    """
+    This is a decorator which can be used to mark functions
+    as deprecated. It will result in a warning being emitted
+    when the function is used.
+    """
+
+    if isinstance(reason, string_types):
+
+        # The @deprecated is used with a 'reason'.
+        #
+        # .. code-block:: python
+        #
+        #    @deprecated("please, use another function")
+        #    def old_function(x, y):
+        #      pass
+
+        def decorator(func1):
+
+            if inspect.isclass(func1):
+                fmt1 = "Call to deprecated class {name} ({reason})."
+            else:
+                fmt1 = "Call to deprecated function {name} ({reason})."
+
+            @functools.wraps(func1)
+            def new_func1(*args, **kwargs):
+                warnings.simplefilter("always", DeprecationWarning)
+                warnings.warn(
+                    fmt1.format(name=func1.__name__, reason=reason),
+                    category=DeprecationWarning,
+                    stacklevel=2,
+                )
+                warnings.simplefilter("default", DeprecationWarning)
+                return func1(*args, **kwargs)
+
+            return new_func1
+
+        return decorator
+
+    elif inspect.isclass(reason) or inspect.isfunction(reason):
+
+        # The @deprecated is used without any 'reason'.
+        #
+        # .. code-block:: python
+        #
+        #    @deprecated
+        #    def old_function(x, y):
+        #      pass
+
+        func2 = reason
+
+        if inspect.isclass(func2):
+            fmt2 = "Call to deprecated class {name}."
+        else:
+            fmt2 = "Call to deprecated function {name}."
+
+        @functools.wraps(func2)
+        def new_func2(*args, **kwargs):
+            warnings.simplefilter("always", DeprecationWarning)
+            warnings.warn(
+                fmt2.format(name=func2.__name__),
+                category=DeprecationWarning,
+                stacklevel=2,
+            )
+            warnings.simplefilter("default", DeprecationWarning)
+            return func2(*args, **kwargs)
+
+        return new_func2
+
+    else:
+        raise TypeError(repr(type(reason)))
+
+
+def convert_to_float_or_exit(putative_float: str, line_info: LineInfo) -> float:
+    """
+    convert input string to a float or call exit_error with a reasonable error message
+    :param putative_float: the string that could be a float
+    :param line_info: line info of where the string came from for error message
+    :return: the string converted to a float
+    """
+    if not is_float(putative_float):
+        msg = f"""
+                the text {putative_float} could not be converted to a float, at
+                line: {line_info.line_no} in
+                file: {line_info.file_name} the line was
+                line: {line_info.line}
+            """
+        exit_error(msg)
+    return float(putative_float)
+
+
+def fnmatch_one_of(target: str, patterns: Tuple[str, ...]) -> bool:
+    """
+    Check if a string code matches any of the patterns
+
+    Args:
+        target: the string to match
+        patterns: a tuple of patterns to match using fnmatch
+
+    Returns:
+        True if the chain code matches any of the patterns
+    """
+    return any(fnmatch(target, pattern) for pattern in patterns)
+
+
+def exit_if_file_has_bytes_and_no_force(output_file: Path, force: bool):
+    """
+    Exit if the output file exists is not empty and the force flag is not set
+
+    Args:
+        output_file: the output file
+        force: if the file exists and isn't empty it still isn't an error if this flag is set
+    """
+    if output_file != STDOUT:
+        if file_exists_and_has_bytes(output_file) and not get_global("force", force):
+            msg = f"""
+            the file {output_file} already exists and is not empty, if you want to overwrite it use the --force flag
+            """
+            exit_error(msg)
+
+
+def file_exists_and_has_bytes(output_file: Path):
+    """
+    Check if a file exists and has bytes in it
+
+    Args:
+        output_file: the file to check
+
+    Returns:
+        True if the file exists and has bytes in it
+    """
+    return (
+        output_file.exists()
+        and output_file.is_file()
+        and output_file.stat().st_size != 0
+    )
