@@ -76,26 +76,36 @@ def main():
     try:
         # import components which will self register, this could and will be automated
         modules = [
-            "nef_pipelines.tools.about",
+            "nef_pipelines.tools.help",
             "nef_pipelines.tools.chains",
             "nef_pipelines.tools.entry",
+            "nef_pipelines.tools.fit",
             "nef_pipelines.tools.frames",
+            "nef_pipelines.tools.globals",
             "nef_pipelines.tools.header",
             "nef_pipelines.tools.peaks",
+            "nef_pipelines.tools.save",
+            "nef_pipelines.tools.series",
             "nef_pipelines.tools.shifts",
+            "nef_pipelines.tools.simulate",
+            "nef_pipelines.tools.sink",
             "nef_pipelines.tools.stream",
             "nef_pipelines.tools.test",
             "nef_pipelines.transcoders.csv",
             "nef_pipelines.transcoders.echidna",
             "nef_pipelines.transcoders.fasta",
             "nef_pipelines.transcoders.mars",
+            "nef_pipelines.transcoders.modelfree",
             "nef_pipelines.transcoders.nmrpipe",
             "nef_pipelines.transcoders.nmrview",
             "nef_pipelines.transcoders.pales",
-            "nef_pipelines.transcoders.pdbx",
+            "nef_pipelines.transcoders.rcsb",
             "nef_pipelines.transcoders.rpf",
             "nef_pipelines.transcoders.shifty",
+            "nef_pipelines.transcoders.shiftx2",
             "nef_pipelines.transcoders.sparky",
+            "nef_pipelines.transcoders.nmrstar",
+            "nef_pipelines.transcoders.talos",
             "nef_pipelines.transcoders.xcamshift",
             "nef_pipelines.transcoders.xeasy",
             "nef_pipelines.transcoders.xplor",
