@@ -6,12 +6,13 @@ from nef_pipelines.lib.test_lib import (
     assert_lines_match,
     isolate_frame,
     path_in_test_data,
+    read_test_data,
     run_and_report,
 )
 from nef_pipelines.lib.util import get_version
 from nef_pipelines.transcoders.nmrpipe.importers.sequence import sequence
 
-HEADER = open(path_in_test_data(__file__, "test_header_entry.txt")).read()
+HEADER = read_test_data("test_header_entry.txt", __file__)
 
 app = typer.Typer()
 app.command()(sequence)
@@ -101,7 +102,6 @@ save_nef_nmr_meta_data
    _nef_nmr_meta_data.program_version  {get_version()}
    _nef_nmr_meta_data.creation_date    2012-01-14T12:00:01.123456
    _nef_nmr_meta_data.uuid             NEFPipelines-2012-01-14T12:00:01.123456-1043321819
-   _nef_nmr_meta_data.creation_time    2012-01-14T12:00:01.123456
 
    loop_
       _nef_run_history.run_number
@@ -123,7 +123,7 @@ def test_pipe_header(fixed_seed):
 
     path = path_in_test_data(__file__, "3aa.tab")
     path_header = path_in_test_data(__file__, "test_header_entry.txt")
-    result = run_and_report(app, ["--pipe", path_header, path])
+    result = run_and_report(app, ["--input", path_header, path])
 
     assert result.exit_code == 0
 
