@@ -1,6 +1,5 @@
 import typer
 
-import nef_pipelines
 from nef_pipelines import nef_app
 
 frames_app = typer.Typer()
@@ -15,7 +14,9 @@ if nef_app.app:
 
     # import of specific importers must be after app creation to avoid circular imports
     import nef_pipelines.tools.frames.delete  # noqa: F401
+    import nef_pipelines.tools.frames.filter  # noqa: F401
     import nef_pipelines.tools.frames.insert  # noqa: F401
     import nef_pipelines.tools.frames.list  # noqa: F401
     import nef_pipelines.tools.frames.rename  # noqa: F401
     import nef_pipelines.tools.frames.tabulate  # noqa: F401
+    import nef_pipelines.tools.frames.unassign  # noqa: F401
