@@ -12,6 +12,8 @@ from tabulate import tabulate
 from nef_pipelines.lib.sequence_lib import (
     MoleculeTypes,
     make_chunked_sequence_1let,
+    offset_chain_residues,
+    sequence_3let_to_sequence_residues,
     translate_1_to_3,
 )
 from nef_pipelines.lib.structures import (
@@ -25,7 +27,7 @@ from nef_pipelines.lib.structures import (
     ShiftData,
     ShiftList,
 )
-from nef_pipelines.lib.util import is_int
+from nef_pipelines.lib.util import exit_error, is_int
 
 
 class PEAK_TYPES(IntEnum):
@@ -354,6 +356,17 @@ def select_records(
     return result
 
 
+def select_data_records(gdb: DbFile, type: str) -> List[DbRecord]:
+    """
+    Select data records from the db file that have the specific type
+
+    :param gdb:  a parsed GBD files
+    :param type: the type of data record to select
+    :return: a list of matching DbRecords
+    """
+    return select_records(gdb, "DATA", predicate=lambda rec: rec.values[0] == type)
+
+
 def gdb_to_3let_sequence(
     gdb: DbFile, molecule_type: MoleculeTypes = MoleculeTypes.PROTEIN
 ) -> List[SequenceResidue]:
@@ -367,9 +380,7 @@ def gdb_to_3let_sequence(
     Returns List[SequenceResidue]:
         a list of sequence residues
     """
-    sequence_records = select_records(
-        gdb, "DATA", predicate=lambda rec: rec.values[0] == "SEQUENCE"
-    )
+    sequence_records = select_data_records(gdb, "SEQUENCE")
 
     sequence = [record.values[1:] for record in sequence_records]
 
@@ -379,7 +390,48 @@ def gdb_to_3let_sequence(
     return translate_1_to_3(sequence_string, molecule_type=molecule_type)
 
 
-def _assignments_to_atom_labels(assignments, dimensions, chain_code="A"):
+def gdb_to_chain_start(gdb_file: DbFile) -> int:
+    """
+    Find a first resid record in a GbdFile and return it or return 1
+    :param gdb_file: a GBD file
+    :return: a chain start
+    """
+
+    chain_start_records = select_data_records(gdb_file, "FIRST_RESID")
+
+    if len(chain_start_records) == 0:
+        chain_start = 1
+    else:
+        chain_start = int(chain_start_records[-1].values[1])
+
+    return chain_start
+
+
+def gdb_to_sequence(gdb_file: DbFile, chain_code: str) -> List[SequenceResidue]:
+
+    """
+    Read the sequence from a GDB file, note if a FIRST_RESID record is found the sequence
+    is offset correctly...
+    :param gdb_file:  a GDB file
+    :param chain_code: the chain code to give the sequence
+    :return: a List of sequence residues
+    """
+    chain_start = gdb_to_chain_start(gdb_file)
+
+    sequence_3let = gdb_to_3let_sequence(gdb_file)
+
+    sequence_residues = sequence_3let_to_sequence_residues(
+        sequence_3let, chain_code=chain_code
+    )
+
+    sequence_residues = offset_chain_residues(
+        sequence_residues, {chain_code: chain_start - 1}
+    )
+
+    return sequence_residues
+
+
+def _assignments_to_atom_labels(assignments, dimensions, chain_code):
     result = []
 
     for assignment in assignments:
@@ -466,7 +518,7 @@ def _get_axis_labels(gdb_file):
     return result
 
 
-def get_gdb_columns(gdb_file):
+def get_gdb_columns(gdb_file: DbFile) -> List[str]:
     return select_records(gdb_file, VARS)[0].values
 
 
@@ -484,7 +536,7 @@ def check_is_shifts_file(gdb_file):
     return expected_fields.issubset(columns)
 
 
-def read_peak_file(gdb_file, args):
+def read_peak_file(gdb_file, chain_code, filter_noise=False):
     data = select_records(gdb_file, VALUES)
 
     dimensions = _get_peak_list_dimension(gdb_file)
@@ -501,15 +553,16 @@ def read_peak_file(gdb_file, args):
         peak = {}
         raw_peaks.append(peak)
         peak_type = line.values[column_indices["TYPE"]]
-        if args.filter_noise and peak_type != PEAK_TYPES.PEAK:
+        if filter_noise and peak_type != PEAK_TYPES.PEAK:
             continue
 
         assignment = line.values[column_indices["ASS"]]
         assignments = assignment.split("-")
         assignments = _propagate_assignments(assignments)
-        assignments = _assignments_to_atom_labels(assignments, dimensions)
+        assignments = _assignments_to_atom_labels(assignments, dimensions, chain_code)
 
         height = line.values[column_indices["HEIGHT"]]
+        # TODO: sort out height errors
         # height_error = line.values[column_indices["DHEIGHT"]]
         # height_percentage_error = height_error / height
         volume = line.values[column_indices["VOL"]]
@@ -555,7 +608,7 @@ def read_peak_file(gdb_file, args):
     return peak_list
 
 
-def read_shift_file(gdb_file, chain_code="A"):
+def read_shift_file(gdb_file, chain_code):
     data = select_records(gdb_file, VALUES)
 
     column_indices = get_column_indices(gdb_file)
@@ -674,15 +727,52 @@ class DataBeforeFormat(BadNmrPipeFile):
     pass
 
 
-def print_pipe_sequence(sequence_1_let: List[str]) -> str:
+def format_pipe_sequence(sequence_1_let: List[str]) -> str:
 
     """
     convert a set of 1 letter amino acid codes to an nmr pipe DATA SEQUENCE record
-    :param sequence_1_let:  1 letter aino acid codes as a list of  strings
-    :return: nmrpipe data sequence string nicely formatted
+    :param sequence_1_let:  1 letter amino acid codes as a list of strings
+    :return: nmrpipe data sequence string nicely formatted as a list of data record strings
     """
 
     row_strings = make_chunked_sequence_1let(sequence_1_let)
 
+    result = []
     for row_string in row_strings:
-        print(f"DATA SEQUENCE {row_string}")
+        result.append(f"DATA SEQUENCE {row_string}")
+
+    return result
+
+
+def exit_if_required_columns_missing(
+    gdb_records: DbFile, required_columns: List[str], file_name: str
+):
+    gdb_columns = get_gdb_columns(gdb_records)
+    column_set = set(gdb_columns)
+
+    required_column_set = set(required_columns)
+
+    if not required_column_set.issubset(column_set):
+        msg = f"""
+            the required column in the file {file_name} are:
+
+            {', '.join(required_columns)}
+
+            found columns are
+
+            {', '.join(gdb_columns)}
+
+            missing columns are
+
+            {', '.join(required_column_set - column_set)}
+        """
+
+        exit_error(msg)
+
+
+def dbfile_to_first_residue_number(gdb_records: DbFile) -> int:
+    first_residue_records = select_data_records(gdb_records, "FIRST_RESID")
+    first_residue = 1
+    if len(first_residue_records) == 1:
+        first_residue = int(first_residue_records[0].values[1])
+    return first_residue
