@@ -6,21 +6,22 @@ from pynmrstar import Entry, Saveframe
 
 from nef_pipelines.lib.constants import NEF_PIPELINES
 from nef_pipelines.lib.nef_lib import read_entry_from_file_or_stdin_or_exit_error
-from nef_pipelines.lib.sequence_lib import chain_code_iter, sequence_from_entry_or_exit
+from nef_pipelines.lib.sequence_lib import (
+    get_chain_code_iter,
+    sequence_from_entry_or_exit,
+)
 from nef_pipelines.lib.shift_lib import shifts_to_nef_frame
 from nef_pipelines.lib.structures import SequenceResidue
 from nef_pipelines.lib.util import (
     STDIN,
-    cached_file_stream,
     exit_error,
     fixup_metadata,
-    get_pipe_file_text_or_exit,
     get_version,
     parse_comma_separated_options,
     script_name,
 )
 from nef_pipelines.transcoders.nmrview import import_app
-from nef_pipelines.transcoders.nmrview.nmrview_lib import parse_shifts, read_sequence
+from nef_pipelines.transcoders.nmrview.nmrview_lib import parse_shifts
 
 app = typer.Typer()
 
@@ -39,7 +40,7 @@ def shifts(
     frame_name: str = typer.Option(
         "nmrview", "-f", "--frame-name", help="a name for the frame"
     ),
-    input: Path = typer.Option(
+    input_path: Path = typer.Option(
         STDIN,
         "-i",
         "--input",
@@ -53,7 +54,7 @@ def shifts(
     """convert nmrview shift file <nmrview-shifts>.out to NEF"""
 
     try:
-        entry = read_entry_from_file_or_stdin_or_exit_error(input)
+        entry = read_entry_from_file_or_stdin_or_exit_error(input_path)
 
         chain_codes = parse_comma_separated_options(chain_codes)
 
@@ -69,9 +70,10 @@ def pipe(entry, chain_codes, sequence, entry_name, file_names):
 
     nmrview_frames = []
 
-    for file_name, chain_code in zip(file_names, chain_code_iter(chain_codes)):
+    chain_code_iter = get_chain_code_iter(chain_codes)
+    for file_name, chain_code in zip(file_names, chain_code_iter):
 
-        with cached_file_stream(file_name) as lines:
+        with open(file_name) as lines:
 
             chain_seqid_to_type = _sequence_to_residue_type_lookup(sequence)
 
@@ -152,31 +154,6 @@ def sequence_from_frames(frames: Saveframe) -> List[SequenceResidue]:
     return residues
 
 
-# TODO this should be replaced by a library function from nef or sequence utils...
-# also need ro report what file we are trying to read shifts from
-def _get_sequence_or_exit(args):
-    sequence_file = None
-    if "sequence" in args:
-        sequence_file = args.sequence
-
-    sequence = None
-    if not sequence_file:
-        try:
-            lines = get_pipe_file_text_or_exit(args)
-
-            entry = Entry.from_string(lines)
-            frames = entry.get_saveframes_by_category("nef_molecular_system")
-            sequence = sequence_from_frames(frames)
-
-        except Exception as e:
-            exit_error(f"failed to read sequence from input stream because {e}", e)
-
-    else:
-        with open(sequence_file, "r") as lines:
-            sequence = read_sequence(lines, chain_code=args.chain_code)
-    return sequence
-
-
 def _sequence_to_residue_type_lookup(
     sequence: List[SequenceResidue],
 ) -> Dict[Tuple[str, int], str]:
