@@ -22,10 +22,11 @@ from pyparsing import (
     ZeroOrMore,
     alphanums,
     nums,
+    rest_of_line,
 )
 from pyparsing.common import pyparsing_common as ppc
 
-from nef_pipelines.lib.nef_lib import UNUSED, PotentialTypes
+from nef_pipelines.lib.nef_lib import UNUSED, PotentialTypes, create_nef_save_frame
 from nef_pipelines.lib.sequence_lib import ANY_CHAIN, replace_chain_in_atom_labels
 from nef_pipelines.lib.structures import (
     AtomLabel,
@@ -69,6 +70,9 @@ from nef_pipelines.lib.util import (
 LEFT_PARENTHESIS = "("
 RIGHT_PARENTHESIS = ")"
 
+SINGLE_QUOTE = "'"
+DOUBLE_QUOTE = '"'
+
 ASSIGN = "assign"
 NEF_DIHEDRAL_RESTRAINT = "nef_dihedral_restraint"
 XPLOR_COMMENT_TOKEN = "!"
@@ -98,7 +102,7 @@ RESIDUE_FACTOR = "residue-factor"
 ATOM_FACTOR = "atom-factor"
 
 RESIDUE_LITERAL = "residue"
-SEGIMENT_IDENTIFIER_LITERAL = "segidentifier"
+SEGMENT_IDENTIFIER_LITERAL = "segidentifier"
 ATOM_NAME_LITERAL = "name"
 
 ATOM_WILDCARDS = "*%#+"
@@ -133,6 +137,8 @@ WEIGHT = "weight"
 TARGET_VALUE = "target_value"
 LOWER_LIMIT = "lower_limit"
 UPPER_LIMIT = "upper_limit"
+EXTRA_INFO = "extra_info"
+CCPN_COMMENT = "ccpn_comment"
 
 DISTANCE_RESTRAINT_TAGS = [
     INDEX,
@@ -190,10 +196,12 @@ _l_paren = Suppress(LEFT_PARENTHESIS)
 _r_paren = Suppress(RIGHT_PARENTHESIS)
 
 
-def _expand_literal(literal, min_length=4, case_insenitive=True):
+def _expand_literal(literal, min_length=4, case_insensitive=True):
     result = []
     for i in range(min_length, len(literal) + 1):
-        elem = CaselessLiteral(literal[:i]) if case_insenitive else Literal(literal[:i])
+        elem = (
+            CaselessLiteral(literal[:i]) if case_insensitive else Literal(literal[:i])
+        )
         result.append(elem)
 
     result.reverse()
@@ -258,7 +266,7 @@ class _NamedTokens(UserList):
         return f"{self.name} {tok_strings}"
 
 
-def _as_named_single_token(name, s, loc, toks):
+def _as_named_single_token(name, _, _1, toks):
     return _NamedToken(name, toks[0])
 
 
@@ -276,16 +284,16 @@ _residue_factor.set_parse_action(_named_resid)
 
 # atom factor
 _atom_literal = Suppress(CaselessLiteral(ATOM_NAME_LITERAL))
-_atom_label = Word(alphanums + ATOM_WILDCARDS).set_results_name(
-    ATOM, list_all_matches=True
-)
+_atom_label = Word(
+    alphanums + ATOM_WILDCARDS + SINGLE_QUOTE + DOUBLE_QUOTE
+).set_results_name(ATOM, list_all_matches=True)
 _atom_factor = (_atom_literal + _atom_label)(ATOM_FACTOR)
 
 _named_atom = partial(_as_named_single_token, ATOM)
 _atom_factor.set_parse_action(_named_atom)
 
 # segment factor
-_segid_literal = Suppress(_expand_literal(SEGIMENT_IDENTIFIER_LITERAL))
+_segid_literal = Suppress(_expand_literal(SEGMENT_IDENTIFIER_LITERAL))
 _segid_label = Word(alphanums, max=4)
 _segid_factor = (_segid_literal + _segid_label)(SEGMENT_FACTOR)
 
@@ -294,7 +302,7 @@ _named_segid = partial(_as_named_single_token, SEGID)
 _segid_factor.setParseAction(_named_segid)
 
 
-def _as_named_tokens(name, s, loc, toks):
+def _as_named_tokens(name, _, _1, toks):
     return _NamedTokens(name, toks)
 
 
@@ -334,6 +342,7 @@ _distance_restraint = Group(
     + ppc.number.set_results_name(DISTANCE).setParseAction(ppc.convertToFloat)
     + ppc.number.set_results_name(DISTANCE_MINUS).setParseAction(ppc.convertToFloat)
     + ppc.number.set_results_name(DISTANCE_PLUS).setParseAction(ppc.convertToFloat)
+    + rest_of_line.set_results_name(EXTRA_INFO)
 )
 
 _distance_restraints = OneOrMore(_distance_restraint)
@@ -362,6 +371,7 @@ def _get_single_atom_selection(
 
     selections = _parse_result_to_atom_selections(parsed)
 
+    selection = None
     if len(selections) != 1:
         valid = False
         invalid_reason = f"there should only be one selection i got {selections}"
@@ -407,7 +417,9 @@ def _get_single_atom_selection(
         else:
             valid = False
             residue_type = None
-            invalid_reason = f"the residue type for chain code: {chain_code} sequence code {sequence_code} is not known"
+            # TODO print out sequence...
+            invalid_reason = f"""the residue type for chain code: {chain_code} sequence code {sequence_code} is not
+                                 known, check this residue is present in you input sequence!"""
 
     if valid:
         residue = SequenceResidue(chain_code, sequence_code, residue_type)
@@ -415,7 +427,7 @@ def _get_single_atom_selection(
 
     if not valid:
         raise XPLORParseException(
-            f"Couldn't get a single restraint from {parsed} because {invalid_reason}"
+            f"Couldn't get a single restraint from {parsed} \n because {invalid_reason}"
         )
 
     return result
@@ -431,7 +443,7 @@ def read_distance_restraints_or_exit_error(
     read a list of dihedral restraints from a file or stream or exit
 
     :param file_path: the path of the file Path('-') indicates stdin
-    :param residue_name_lookup:  a dictionary of residue names keyes on chain_code, residue_code
+    :param residue_name_lookup:  a dictionary of residue names keys on chain_code, residue_code
     :param chain_code: a chain code to use if no chain code is specified or use_chains is True
     :param use_chains: use the passed in chain_code rather than any read segids
     :return: a list of dihedral restraints
@@ -443,13 +455,16 @@ def read_distance_restraints_or_exit_error(
 
     file_path_display_name = get_display_file_name(file_path)
 
-    restraints = parse_distance_restraints(
-        restraint_text,
-        residue_name_lookup,
-        file_path_display_name,
-        chain_code,
-        use_chains,
-    )
+    try:
+        restraints = parse_distance_restraints(
+            restraint_text,
+            residue_name_lookup,
+            file_path_display_name,
+            chain_code,
+            use_chains,
+        )
+    except XPLORParseException as e:
+        exit_error("there was an error", e)
 
     return restraints
 
@@ -465,16 +480,29 @@ def distance_restraints_to_nef(
     """
     loop = Loop.from_scratch(category="nef_distance_restraint")
 
+    have_comments = False
+    for restraint in restraints:
+        if restraint.comment is not None:
+            have_comments = True
+            break
+
     for tag in DISTANCE_RESTRAINT_TAGS:
         loop.add_tag(tag)
 
+    if have_comments:
+        loop.add_tag(CCPN_COMMENT)
+
     for i, restraint in enumerate(restraints, start=1):
         selection_1 = restraint.atom_list_1[0]
         selection_2 = restraint.atom_list_2[0]
         d = restraint.target_distance
         d_minus = restraint.distance_minus
         d_plus = restraint.distance_plus
-
+        comment = (
+            restraint.comment
+            if restraint.comment is not None or restraint.comment != ""
+            else UNUSED
+        )
         row = {
             INDEX: i,
             RESTRAINT_ID: i,
@@ -487,18 +515,17 @@ def distance_restraints_to_nef(
             ATOM_2: selection_2.atom_name,
             WEIGHT: 1.0,
             TARGET_VALUE: round(d, DEFAULT_PRECISION),
-            LOWER_LIMIT: round(d - d_minus, DEFAULT_PRECISION),
-            UPPER_LIMIT: round(d + d_plus, DEFAULT_PRECISION),
+            LOWER_LIMIT: round(d_minus, DEFAULT_PRECISION),
+            UPPER_LIMIT: round(d_plus, DEFAULT_PRECISION),
         }
 
+        if have_comments:
+            row[CCPN_COMMENT] = comment
+
         loop.add_data([row])
 
-    save_frame_name = f"nef_distance_restraint_list_{frame_name}"
-
-    save_frame = Saveframe.from_scratch(save_frame_name, "nef_distance_restraint")
-
-    save_frame.add_tag("sf_category", "nef_distance_restraint_list")
-    save_frame.add_tag("sf_framecode", save_frame_name)
+    NEF_DISTANCE_RESTRAINT_LIST = "nef_distance_restraint_list"
+    save_frame = create_nef_save_frame(NEF_DISTANCE_RESTRAINT_LIST, frame_name)
     save_frame.add_tag("potential_type", PotentialTypes.UNDEFINED)
 
     save_frame.add_loop(loop)
@@ -525,7 +552,7 @@ def dihedral_restraints_to_nef(
         selection_2 = restraint.atom_2
         selection_3 = restraint.atom_3
         selection_4 = restraint.atom_4
-        target = restraint.target_angle
+        target = restraint.target_value
         lower_limit = restraint.lower_limit
         upper_limit = restraint.upper_limit
 
@@ -553,12 +580,8 @@ def dihedral_restraints_to_nef(
 
         loop.add_data([row])
 
-    save_frame_name = f"nef_dihedral_restraint_list_{frame_name}"
-
-    save_frame = Saveframe.from_scratch(save_frame_name, "nef_dihedral_restraint")
-
-    save_frame.add_tag("sf_category", "nef_dihedral_restraint_list")
-    save_frame.add_tag("sf_framecode", save_frame_name)
+    NEF_DIHEDRAL_RESTRAINT_LIST = "nef_dihedral_restraint_list"
+    save_frame = create_nef_save_frame(NEF_DIHEDRAL_RESTRAINT_LIST, frame_name)
     save_frame.add_tag("potential_type", PotentialTypes.UNDEFINED)
 
     save_frame.add_loop(loop)
@@ -579,7 +602,7 @@ def _get_approximate_restraint_strings(text: str) -> List[str]:
 
     new_lines = "\n".join(new_lines)
 
-    restraints = new_lines.split(ASSIGN)
+    restraints = list(_expand_literal(ASSIGN).split(new_lines))
 
     if restraints[0] == "":
         restraints = restraints[1:]
@@ -596,8 +619,8 @@ def read_dihedral_restraints_or_exit_error(
     """
     read a list of dihedral restraints from a file or stream or exit
 
-    :param file_path: the poth of the file Path('-') indicates stdin
-    :param residue_name_lookup:  a dictionary of residue names keyes on chain_code, residue_code
+    :param file_path: the path of the file Path('-') indicates stdin
+    :param residue_name_lookup:  a dictionary of residue names keys on chain_code, residue_code
     :param chain_code: a chain code to use if no chain code is specified or use_chains is True
     :param use_chains: use the passed in chain_code rather than any read segids
     :return: a list of dihedral restraints
@@ -672,7 +695,7 @@ def parse_dihedral_restraints(
                 msg = f"""\
                     got a multi atom selection for the {atom_number} atom in restraint number {i}
                     in {file_path_display_name}
-                    dihedral restrainst require single atom selections...
+                    dihedral restraints require single atom selections...
                     the restraint text is most probably:
                 """
                 msg = dedent(msg)
@@ -692,7 +715,7 @@ def parse_dihedral_restraints(
 
         restraint = DihedralRestraint(
             *atom_selections,
-            target_angle=target_angle,
+            target_value=target_angle,
             lower_limit=lower_limit,
             upper_limit=upper_limit,
         )
@@ -707,7 +730,7 @@ def parse_distance_restraints(
     file_path_display_name: str,
     chain_code: str,
     use_chains: bool = False,
-) -> List[DihedralRestraint]:
+) -> List[DistanceRestraint]:
     """
     parse xplor distance restraints into DistanceRestraint structures
 
@@ -753,7 +776,7 @@ def parse_distance_restraints(
                 msg = f"""\
                     got a multi atom selection for the {atom_number} atom in restraint number {i}
                     in {file_path_display_name}
-                    dihedral restrainst require single atom selections...
+                    dihedral restraints require single atom selections...
                     the restraint text is most probably:
                 """
                 msg = dedent(msg)
@@ -769,6 +792,9 @@ def parse_distance_restraints(
         target_distance = restraint.get(DISTANCE)
         distance_minus = restraint.get(DISTANCE_MINUS)
         distance_plus = restraint.get(DISTANCE_PLUS)
+        comment = restraint.get(EXTRA_INFO).strip()
+        if comment == "":
+            comment = None
 
         atom_selections = [[atom_selection] for atom_selection in atom_selections]
         restraint = DistanceRestraint(
@@ -776,6 +802,7 @@ def parse_distance_restraints(
             target_distance=target_distance,
             distance_minus=target_distance - distance_minus,
             distance_plus=target_distance + distance_plus,
+            comment=comment,
         )
 
         restraints.append(restraint)
